@@ -9,13 +9,12 @@ local fsu     = require 'fs-utility'
 local define  = require "proto.define"
 local files   = require 'files'
 local await   = require 'await'
-local timer   = require 'timer'
 local encoder = require 'encoder'
 local ws      = require 'workspace.workspace'
 local scope   = require 'workspace.scope'
 local inspect = require 'inspect'
+local jsonb   = require 'json-beautify'
 local jsonc   = require 'jsonc'
-local json    = require 'json'
 
 local m = {}
 
@@ -189,10 +188,14 @@ local function compileSingleMetaDoc(uri, script, metaLang, status)
     if not suc then
         log.debug('MiddleScript:\n', middleScript)
     end
+    local text = table.concat(compileBuf)
     if disable and status == 'default' then
-        return nil
+        return text, false
     end
-    return table.concat(compileBuf)
+    if status == 'disable' then
+        return text, false
+    end
+    return text, true
 end
 
 local function loadMetaLocale(langID, result)
@@ -223,11 +226,8 @@ local function initBuiltIn(uri)
         loadMetaLocale(langID, metaLang)
     end
 
-    if scp:get('metaPath') == metaPath:string() then
-        log.debug('Has meta path, skip:', metaPath:string())
-        return
-    end
-    scp:set('metaPath', metaPath:string())
+    local metaPaths = {}
+    scp:set('metaPaths', metaPaths)
     local suc = xpcall(function ()
         if not fs.exists(metaPath) then
             fs.create_directories(metaPath)
@@ -242,13 +242,10 @@ local function initBuiltIn(uri)
     for libName, status in pairs(define.BuiltIn) do
         status = config.get(uri, 'Lua.runtime.builtin')[libName] or status
         log.debug('Builtin status:', libName, status)
-        if status == 'disable' then
-            goto CONTINUE
-        end
 
         ---@type fs.path
         local libPath = templateDir / (libName .. '.lua')
-        local metaDoc = compileSingleMetaDoc(uri, fsu.loadFile(libPath), metaLang, status)
+        local metaDoc, include = compileSingleMetaDoc(uri, fsu.loadFile(libPath), metaLang, status)
         if metaDoc then
             metaDoc = encoder.encode(encoding, metaDoc, 'auto')
 
@@ -259,13 +256,17 @@ local function initBuiltIn(uri)
 
             local ok, err = out:saveFile(outputLibName, metaDoc)
             if not ok then
-                log.debug("Save Meta File:", err)
+                log.debug("Save Meta File Failed:", err)
                 goto CONTINUE
             end
 
             local outputPath = metaPath / outputLibName
             m.metaPaths[outputPath:string()] = true
             log.debug('Meta path:', outputPath:string())
+
+            if include then
+                metaPaths[#metaPaths+1] = outputPath:string()
+            end
         end
         ::CONTINUE::
     end
@@ -284,7 +285,7 @@ local function loadSingle3rdConfigFromJson(libraryDir)
         return nil
     end
 
-    local suc, cfg = xpcall(jsonc.decode, function (err)
+    local suc, cfg = xpcall(jsonc.decode_jsonc, function (err)
         log.error('Decode config.json failed at:', libraryDir:string(), err)
     end, configText)
     if not suc then
@@ -339,7 +340,7 @@ local function loadSingle3rdConfig(libraryDir)
         if not cfg then
             return
         end
-        local jsonbuf = json.beautify(cfg)
+        local jsonbuf = jsonb.beautify(cfg)
         client.requestMessage('Info', lang.script.WINDOW_CONFIG_LUA_DEPRECATED, {
             lang.script.WINDOW_CONVERT_CONFIG_LUA,
         }, function (action, index)
@@ -364,7 +365,7 @@ local function loadSingle3rdConfig(libraryDir)
     end
     if cfg.files then
         for i, filename in ipairs(cfg.files) do
-            if plat.OS == 'Windows' then
+            if plat.os == 'windows' then
                 filename = filename:gsub('/', '\\')
             else
                 filename = filename:gsub('\\', '/')
@@ -468,34 +469,45 @@ end
 
 local hasAsked = {}
 ---@async
-local function askFor3rd(uri, cfg)
+local function askFor3rd(uri, cfg, checkThirdParty)
     if hasAsked[cfg.name] then
         return nil
     end
-    hasAsked[cfg.name] = true
-    local yes1 = lang.script.WINDOW_APPLY_WHIT_SETTING
-    local yes2 = lang.script.WINDOW_APPLY_WHITOUT_SETTING
-    local no   = lang.script.WINDOW_DONT_SHOW_AGAIN
-    local result = client.awaitRequestMessage('Info'
-        , lang.script('WINDOW_ASK_APPLY_LIBRARY', cfg.name)
-        , {yes1, yes2, no}
-    )
-    if not result then
-        return nil
-    end
-    if result == yes1 then
+
+    if checkThirdParty == 'Apply' then
         apply3rd(uri, cfg, false)
-    elseif result == yes2 then
+    elseif checkThirdParty == 'ApplyInMemory' then
         apply3rd(uri, cfg, true)
-    else
-        client.setConfig({
-            {
-                key    = 'Lua.workspace.checkThirdParty',
-                action = 'set',
-                value  = false,
-                uri    = uri,
-            },
-        }, false)
+    elseif checkThirdParty == 'Disable' then
+        return nil
+    elseif checkThirdParty == 'Ask' then
+        hasAsked[cfg.name] = true
+        local applyAndSetConfig = lang.script.WINDOW_APPLY_WHIT_SETTING
+        local applyInMemory = lang.script.WINDOW_APPLY_WHITOUT_SETTING
+        local dontShowAgain   = lang.script.WINDOW_DONT_SHOW_AGAIN
+        local result = client.awaitRequestMessage('Info'
+            , lang.script('WINDOW_ASK_APPLY_LIBRARY', cfg.name)
+            , {applyAndSetConfig, applyInMemory, dontShowAgain}
+        )
+        if not result then
+            -- "If none got selected"
+            -- See: https://microsoft.github.io/language-server-protocol/specifications/lsp/3.17/specification/#window_showMessageRequest
+            return nil
+        end
+        if result == applyAndSetConfig then
+            apply3rd(uri, cfg, false)
+        elseif result == applyInMemory then
+            apply3rd(uri, cfg, true)
+        else
+            client.setConfig({
+                {
+                    key    = 'Lua.workspace.checkThirdParty',
+                    action = 'set',
+                    value  = 'Disable',
+                    uri    = uri,
+                },
+            }, false)
+        end
     end
 end
 
@@ -516,7 +528,7 @@ local function wholeMatch(a, b)
     return true
 end
 
-local function check3rdByWords(uri, configs)
+local function check3rdByWords(uri, configs, checkThirdParty)
     if not files.isLua(uri) then
         return
     end
@@ -542,9 +554,10 @@ local function check3rdByWords(uri, configs)
             for _, word in ipairs(cfg.words) do
                 await.delay()
                 if wholeMatch(text, word) then
+                    log.info('Found 3rd library by word: ', word, uri, library, inspect(config.get(uri, 'Lua.workspace.library')))
                     ---@async
                     await.call(function ()
-                        askFor3rd(uri, cfg)
+                        askFor3rd(uri, cfg, checkThirdParty)
                     end)
                     return
                 end
@@ -554,7 +567,7 @@ local function check3rdByWords(uri, configs)
     end, id)
 end
 
-local function check3rdByFileName(uri, configs)
+local function check3rdByFileName(uri, configs, checkThirdParty)
     local path = ws.getRelativePath(uri)
     if not path then
         return
@@ -570,12 +583,17 @@ local function check3rdByFileName(uri, configs)
             if hasAsked[cfg.name] then
                 goto CONTINUE
             end
+            local library = ('%s/library'):format(cfg.dirname)
+            if util.arrayHas(config.get(uri, 'Lua.workspace.library'), library) then
+                goto CONTINUE
+            end
             for _, filename in ipairs(cfg.files) do
                 await.delay()
                 if wholeMatch(path, filename) then
+                    log.info('Found 3rd library by filename: ', filename, uri, library, inspect(config.get(uri, 'Lua.workspace.library')))
                     ---@async
                     await.call(function ()
-                        askFor3rd(uri, cfg)
+                        askFor3rd(uri, cfg, checkThirdParty)
                     end)
                     return
                 end
@@ -590,8 +608,12 @@ local function check3rd(uri)
     if ws.isIgnored(uri) then
         return
     end
-    if not config.get(uri, 'Lua.workspace.checkThirdParty') then
+    local checkThirdParty = config.get(uri, 'Lua.workspace.checkThirdParty')
+    -- Backwards compatability: `checkThirdParty` used to be a boolean.
+    if not checkThirdParty or checkThirdParty == 'Disable' then
         return
+    elseif checkThirdParty == true then
+        checkThirdParty = 'Ask'
     end
     local scp = scope.getScope(uri)
     if not scp:get 'canCheckThirdParty' then
@@ -601,8 +623,8 @@ local function check3rd(uri)
     if not thirdConfigs then
         return
     end
-    check3rdByWords(uri, thirdConfigs)
-    check3rdByFileName(uri, thirdConfigs)
+    check3rdByWords(uri, thirdConfigs, checkThirdParty)
+    check3rdByFileName(uri, thirdConfigs, checkThirdParty)
 end
 
 local function check3rdOfWorkspace(suri)
